#include <avr/pgmspace.h>

#include "ac_defines.h"
#include "ac_hitachi.h"

// TODO typedefs
volatile uint8* LCDValue;
volatile uint8* LCDRead;
volatile uint8* LCDStatus;
volatile uint8* LCDAddressLowByte;
volatile uint8* LCDAddressHighByte;
volatile int LCDPageAddress;


void HWL_InitializeDisplay(void)
{
	LCDValue  = (uint8*)0x8002;
	LCDAddressLowByte  = (uint8*)0x8000;
	LCDAddressHighByte = (uint8*)0x8001;
	LCDRead   = (uint8*)0x8003;
	LCDStatus = (uint8*)0x8001;

	LCDPageAddress = 0;
	*LCDAddressHighByte = 0;
	*LCDAddressLowByte = 0;
}

void HWL_SetPixel(int x, int y)
{
	unsigned char value;
	unsigned char pixel = x % 8;
	int address = LCDPageAddress + y * HWL_PageByteWidth + x/8;

	// set write address
	*LCDAddressHighByte = (uint8)(address >> 8);
	*LCDAddressLowByte  = (uint8)(address);

	// read current byte
	*LCDRead = 0xff;
	while (*LCDStatus & 0x80);
	value = *LCDValue;

	// calculate pixel address and or combine with current byte
	pixel = pixel == 0 ? 0x80 : 0x80 >> pixel;
	*LCDValue = pixel | value;
	while (*LCDStatus & 0x80);
}

unsigned char HWL_GetByte(int address)
{
	// set write address
	*LCDAddressHighByte = (uint8)(address >> 8);
	*LCDAddressLowByte  = (uint8)(address);

	// read current byte
	*LCDRead = 0xff;
	while (*LCDStatus & 0x80);
	return *LCDValue;
}

void HWL_ClearScreen(void)
{
	int address = LCDPageAddress;

	do
	{
		*LCDAddressHighByte = (uint8)(address >> 8);
		*LCDAddressLowByte  = (uint8)(address);
		while (*LCDStatus & 0x80);
		*LCDValue = 0;
		while (*LCDStatus & 0x80);

		address++;
	} while (address < HWL_PageByteWidth * HWL_PixelHeight);
}

